package ru.octol1ttle.knockdowns.client.gui;

import java.util.Map;
import net.minecraft.client.Minecraft;
import net.minecraft.client.gui.FontRenderer;
import net.minecraft.client.gui.ScaledResolution;
import net.minecraft.client.renderer.GlStateManager;
import net.minecraft.client.resources.I18n;
import net.minecraft.client.settings.KeyBinding;
import net.minecraft.entity.Entity;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.math.MathHelper;
import net.minecraft.util.math.Vec3d;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import org.lwjgl.input.Keyboard;
import org.lwjgl.opengl.GL11;
import ru.octol1ttle.knockdowns.Tags;
import ru.octol1ttle.knockdowns.client.communication.CalloutManager;
import ru.octol1ttle.knockdowns.client.event.KnockdownsKeyListener;
import ru.octol1ttle.knockdowns.client.util.Callout;

@SideOnly(Side.CLIENT)
public class CommunicationGui extends KnockdownsBaseGui {
    private static final Minecraft client = Minecraft.getMinecraft();
    private static final ResourceLocation LEFT_ARROW = new ResourceLocation(Tags.MOD_ID, "textures/gui/left_arrow.png");
    private static final ResourceLocation DOWN_ARROW = new ResourceLocation(Tags.MOD_ID, "textures/gui/down_arrow.png");
    private static final ResourceLocation UP_ARROW = new ResourceLocation(Tags.MOD_ID, "textures/gui/up_arrow.png");
    private static final ResourceLocation RIGHT_ARROW = new ResourceLocation(Tags.MOD_ID, "textures/gui/right_arrow.png");
    private static final int SCREEN_EDGE_MARGIN = 5;
    private static final int SEPARATOR_MARGIN = 2;
    private static final int KEY_SIZE = 17;
    private float totalPartialTicks;

    @Override
    public void render(float partialTicks, ScaledResolution resolution) {
        GlStateManager.pushMatrix();
        float scale = 1.0f;
        if (resolution.getScaleFactor() > 3) {
            scale = (float) 3 / resolution.getScaleFactor();
            GlStateManager.scale(scale, scale, 1.0f);
        }
        FontRenderer font = client.fontRenderer;

        int x = SCREEN_EDGE_MARGIN;
        int y = (int) (resolution.getScaledHeight() / scale - SCREEN_EDGE_MARGIN - font.FONT_HEIGHT);

        KeyBinding[] sortedBindings = new KeyBinding[4];
        for (KeyBinding binding : KnockdownsKeyListener.calloutBindings.keySet())
        {
            switch (binding.getKeyCode()) {
                case Keyboard.KEY_LEFT:
                    sortedBindings[0] = binding;
                    break;
                case Keyboard.KEY_DOWN:
                    sortedBindings[1] = binding;
                    break;
                case Keyboard.KEY_UP:
                    sortedBindings[2] = binding;
                    break;
                case Keyboard.KEY_RIGHT:
                    sortedBindings[3] = binding;
                    break;
            }
        }

        KeyBinding leftCallout = sortedBindings[0];
        if (leftCallout != null) {
            String text = I18n.format(KnockdownsKeyListener.calloutBindings.get(leftCallout).get().getTextKey());
            font.drawStringWithShadow(
                    text,
                    x,
                    y - 12,
                    0xFFFFFF
            );

            x += font.getStringWidth(text) + SEPARATOR_MARGIN;
            client.getTextureManager().bindTexture(LEFT_ARROW);
            this.drawTexture(
                    x,
                    y - KEY_SIZE,
                    KEY_SIZE,
                    KEY_SIZE
            );
            x += KEY_SIZE + SEPARATOR_MARGIN;
        }

        KeyBinding downCallout = sortedBindings[1];
        if (downCallout != null) {
            String text = I18n.format(KnockdownsKeyListener.calloutBindings.get(downCallout).get().getTextKey());
            font.drawStringWithShadow(
                    text,
                    x + KEY_SIZE * 0.5f - font.getStringWidth(text) * 0.5f,
                    y + SEPARATOR_MARGIN,
                    0xFFFFFF
            );
            client.getTextureManager().bindTexture(DOWN_ARROW);
            this.drawTexture(
                    x,
                    y - KEY_SIZE,
                    KEY_SIZE,
                    KEY_SIZE
            );
        }

        KeyBinding upCallout = sortedBindings[2];
        if (upCallout != null) {
            String text = I18n.format(KnockdownsKeyListener.calloutBindings.get(upCallout).get().getTextKey());
            font.drawStringWithShadow(
                    text,
                    x + KEY_SIZE * 0.5f - font.getStringWidth(text) * 0.5f,
                    y - KEY_SIZE * 2 - 12,
                    0xFFFFFF
            );

            client.getTextureManager().bindTexture(UP_ARROW);
            this.drawTexture(
                    x,
                    y - KEY_SIZE * 2 - 2,
                    KEY_SIZE,
                    KEY_SIZE
            );
        }

        KeyBinding rightCallout = sortedBindings[3];
        if (rightCallout != null) {
            x += KEY_SIZE + SEPARATOR_MARGIN;
            String text = I18n.format(KnockdownsKeyListener.calloutBindings.get(rightCallout).get().getTextKey());
            font.drawStringWithShadow(
                    text,
                    x + KEY_SIZE + SEPARATOR_MARGIN,
                    y - 12,
                    0xFFFFFF
            );

            client.getTextureManager().bindTexture(RIGHT_ARROW);
            this.drawTexture(
                    x,
                    y - KEY_SIZE,
                    KEY_SIZE,
                    KEY_SIZE
            );
        }

        GlStateManager.popMatrix();
    }

    public void renderCallouts(float partialTicks) {
        totalPartialTicks += partialTicks;
        for (Map.Entry<Integer, Callout> calloutEntry : CalloutManager.getCallouts()) {
            Entity entity = client.world.getEntityByID(calloutEntry.getKey());
            renderCallout(
                    I18n.format(calloutEntry.getValue().getType().getTextKey()),
                    entity != null ? entity.getPositionEyes(partialTicks).add(0, 1, 0) : calloutEntry.getValue().getPosition(),
                    client.getRenderManager().playerViewX,
                    client.getRenderManager().playerViewY,
                    client.getRenderManager().options.thirdPersonView == 2
            );
        }
    }

    private void renderCallout(String text, Vec3d position, float pitch, float yaw, boolean isThirdPersonFrontal) {
        FontRenderer font = client.fontRenderer;

        Vec3d deltaPos = position.subtract(client.getRenderManager().viewerPosX, client.getRenderManager().viewerPosY, client.getRenderManager().viewerPosZ);

        GlStateManager.pushMatrix();
        GlStateManager.translate(deltaPos.x, deltaPos.y, deltaPos.z);
        GlStateManager.rotate(-yaw, 0.0F, 1.0F, 0.0F);
        GlStateManager.rotate((float)(isThirdPersonFrontal ? -1 : 1) * pitch, 1.0F, 0.0F, 0.0F);
        float scale = (float) Math.max(deltaPos.length() / 8.0f, 1.0f) * (0.75f + MathHelper.abs((float) (MathHelper.sin(totalPartialTicks / 10F) / Math.PI)));
        GlStateManager.scale(-0.025F * scale, -0.025F * scale, 0.025F * scale);
        GlStateManager.color(1f, 1f, 1f, 1f);
        GlStateManager.disableCull();
        GlStateManager.depthFunc(GL11.GL_ALWAYS);

        font.drawStringWithShadow(text, -font.getStringWidth(text) * 0.5f, -font.FONT_HEIGHT * 0.5f, 0xFFFFFF);

        GlStateManager.depthFunc(GL11.GL_LEQUAL);
        GlStateManager.enableCull();
        GlStateManager.popMatrix();
    }
}
